/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.element.scoped;

import org.jabhts.translator.compiler.application.baseclass.Application;
import org.jabhts.translator.compiler.application.baseclass.InvalidApplicationException;
import org.jabhts.translator.compiler.application.scope.Scope;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.tree.AbstractEmitter;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;
import org.jabhts.translator.compiler.utils.InternalCompilerException;
import org.w3c.dom.Element;

abstract class AbstractScopeApplication extends Application
{
	protected AbstractScopeApplication(TargetingInfo target) throws CompilerException
	{
		super(target);
		if (target.cls == null)
			throw new InternalCompilerException("Must be applied to class only!");
		if (target.cls.getEnclosingClass() == null)
			throw new InvalidApplicationException("Must not be applied to outer-class but only inner-classes");
	}

	public Scope makeSubScope(Scope outerScope) throws CompilerException
	{
		return outerScope.getSubScope(cls, target, null);
	}

	public void createEmitter(ElementEmitters emitters, Element e) throws CompilerException
	{
		// annot er altid på inner-class.
		final Scope subScope = makeSubScope(scope);
		if (subScope == null)
			return; // we have already logged the error
		createEmitter(emitters, subScope);
	}

	protected abstract void createEmitter(ElementEmitters emitters, Scope subScope) throws CompilerException;

	public abstract static class AbstractScopeEmitter extends AbstractEmitter<Application>
	{
		final Scope subScope;

		protected AbstractScopeEmitter(ElementEmitters emitters, Application handler, Scope subScope)
		{
			super(emitters.program, handler);
			this.subScope = subScope;
		}

		/**
		 * Called before tag has been emitted.
		 */
		public void preElement() throws CompilerException
		{
			program.addCreateScopeInstance(subScope);
		}

		/**
		 * Called after the compiler has processed the element.
		 */
		public void postElement() throws CompilerException
		{
			program.addLeaveScopeInstance(subScope);
		}
	}
}
