/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.codeGeneration.api;

import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Type;
import org.jabhts.translator.compiler.application.scope.Scope;
import org.jabhts.translator.compiler.utils.CompilerUtils;

public class IteratorInfo
{
	public final Scope scope;
	public boolean isIterable;
	public final String description;
	public final String iterVar, itemVar, itemType;

	public IteratorInfo(Scope scope, String description, boolean isIterable, String namePrefix, Type itemType)
	{
		this.scope = scope;
		this.description = description;
		this.isIterable = isIterable;
		this.iterVar = namePrefix + "iter";
		this.itemVar = namePrefix + "item";
		//System.err.println("---------------------");
		String s = toString(itemType);
		//System.err.println("TOSTRING(" + itemType + ") -> " + s);
		this.itemType = CompilerUtils.zapPrefix(s, "class ");
	}

	private static String toString(Type t)
	{
		String s;
		if (t instanceof GenericArrayType) {
			GenericArrayType tt = (GenericArrayType)t;
			s = toString(tt) + "[]";
		} else if (t instanceof Class) {
			Class<?> cc = (Class<?>)t;
			if (cc.isArray())
				s = toString(cc.getComponentType()) + "[]";
			else
				s = cc.getName();
		} else {
			s = t.toString();
		}
		//System.err.println("TYPE(" + t.getClass().getName() + ") -> " + s);
		return s;
	}
}
